﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;

public class DistributeCubes : MonoBehaviour {
    public static string prettyTime(float time, bool brief = false)
    {
        if (float.IsNaN(time)) return "-:--.--";
        int minutes = Mathf.FloorToInt(time / 60);
        int seconds = Mathf.FloorToInt(time - minutes * 60);
        int hunds = Mathf.FloorToInt((time - minutes * 60 - seconds) * 100);
        string output;
        if(brief) output = string.Format("{0}:{1:D2}", minutes, seconds);
        else output = string.Format("{0}:{1:D2}.{2:D2}", minutes, seconds, hunds);
        return output;

    }
    public Transform Room;
    public GameObject cubePrototype;
    public GameObject coinPrototype;
    public int cubesAcross = 7;
    public float cubeChance = 0.25f;
    public float coinChance = 0.25f;
    public int coinCount = 0;
    public int coinTotal;
    private bool canMakeNoise;
    private int initCoinsToAdd = 80;
    private int coinsToAdd;
    private int cubesToAdd = 40;
    public AudioClip[] coinSounds;
    public AudioClip lastCoinSound;
    public AudioClip startSound;
    public AudioClip tickSound;
    private int soundIndex = 0;
    private int[,,] map;
    private float beamChance = 0.25f;
    public GameObject cubeHolderProto;
    private GameObject heldHolder;
    public int currentLevel = 0;
    public float levelTimer;
    public float countdownLength = 1.51f;
    public Text levelText;
    public Text timerText;
    private float finishTimer;
    public bool cleared = false;
    public GameObject finishRippleProto;
    public GameObject normalRippleProto;
    private int countdown;
    private Vector3 playerStartPos;

    private bool hWasZero = false;

    private float pbForLevel;
    private float wrForLevel;
    private bool setPB = false;

    public GameObject player;
    // Use this for initialization

    void updateLevelText()
    {
        if (setPB) levelText.color = new Color(1, 1, 0.5f, 1);
        else levelText.color = Color.white;
        levelText.text = "Level " + currentLevel.ToString();
        if (pbForLevel != 0) levelText.text += " - PB: " + prettyTime(pbForLevel);
        if (wrForLevel != 0) levelText.text += " - WR: " + prettyTime(wrForLevel);
    }
    
    void Start() {
        playerStartPos = player.transform.position;
        buildMap(currentLevel);
        //player = GameObject.FindGameObjectWithTag("Player");
        
    }

    void buildMap(int level){
        pbForLevel = PlayerPrefs.GetFloat("PB" + currentLevel.ToString(), 0);
        wrForLevel = PlayerPrefs.GetFloat("WR" + currentLevel.ToString(), 0);

        setPB = false;
        levelText.color = Color.white;
        SendMessage("ResetRotation");
        player.transform.position = playerStartPos;
        player.GetComponent<Rigidbody>().velocity = Vector3.zero;
        countdown = 3;
        finishTimer = 3;
        cleared = false;
        updateLevelText();
        coinTotal = 0;
        coinCount = 0;
        levelTimer = -countdownLength;
        Random.seed = level;
        coinsToAdd = initCoinsToAdd;
        if (heldHolder) Destroy(heldHolder);

        Room.rotation = Quaternion.identity;

        heldHolder = GameObject.Instantiate<GameObject>(cubeHolderProto);
        heldHolder.transform.parent = Room;

        int spacesLeft = cubesAcross * cubesAcross * cubesAcross;
        
        map = new int[cubesAcross, cubesAcross, cubesAcross];
        for (int x = 0; x < cubesAcross; x++)
            for (int y = 0; y < cubesAcross; y++)
            {
                if (Random.Range(0, 1f) < beamChance)
                {
                    int start = Random.Range(0, cubesAcross);
                    int end = Random.Range(0, cubesAcross);
                    if (start > end) { int tmp = start; start = end; end = tmp; }
                    for (int z = 0; z < cubesAcross; z++)
                    {
                        if (z >= start && z <= end)
                        {
                            int randir = Random.Range(0, 3);
                            if(randir == 0) map[x, y, z] = 1;
                            if (randir == 1) map[y, z, x] = 1;
                            if (randir == 2) map[z, x, y] = 1;
                        }
                    }
                }
            }
        

        float backwards = -(cubesAcross - 1) / 2;
        for (int x = 0; x < cubesAcross; x++)
            for (int y = 0; y < cubesAcross; y++)
                for (int z = 0; z < cubesAcross; z++)
                {
                    Vector3 pos = new Vector3(x + backwards, y + backwards, z + backwards);
                    if(map[x,y,z] == 1)//(Random.Range(0, spacesLeft) < cubesToAdd)//(Random.Range(0f, 1f) < cubeChance)
                    {
                        GameObject cube = (GameObject)GameObject.Instantiate(cubePrototype, pos, Quaternion.identity);
                        cube.transform.parent = heldHolder.transform;
                        cubesToAdd--;
                    }
                    else if (Random.Range(0, spacesLeft) < coinsToAdd)//(Random.Range(0f, 1f) < coinChance)
                    {
                        GameObject coin = (GameObject)GameObject.Instantiate(coinPrototype, pos, Quaternion.identity);
                        coin.transform.parent = heldHolder.transform;
                        coinCount++;
                        coinsToAdd--;
                    }
                    spacesLeft--;
                }

        coinTotal = coinCount;
    }

    void CollectedCoin(Vector3 posat)
    {
        coinCount--;
        if (canMakeNoise)
        {
            //GetComponent<AudioSource>().pitch = 2 - coinCount * 1.0f / coinTotal;
            GetComponent<AudioSource>().PlayOneShot(coinSounds[soundIndex]);
            soundIndex++;
            if (soundIndex >= coinSounds.Length) soundIndex = 0;
            GameObject player = GameObject.FindGameObjectWithTag("Player");
            GameObject.Instantiate(normalRippleProto, new Vector3(posat.x, posat.y, -7), Quaternion.identity);
        }
        canMakeNoise = false;
        if (coinCount == 0)
        {
            cleared = true;
            GetComponent<AudioSource>().PlayOneShot(lastCoinSound);
            GameObject player = GameObject.FindGameObjectWithTag("Player");
            GameObject.Instantiate(finishRippleProto, new Vector3(posat.x, posat.y, -7), Quaternion.identity);

            if (wrForLevel == 0 || levelTimer < wrForLevel)
            {
                string url = "https://script.google.com/macros/s/AKfycbxEcXa7oUauhmvn9Dygrwr3iTfuQDrkLog92G87oJgZLkX-gqKn/exec?mode=submit&level="+currentLevel+"&score="+levelTimer;
                WWW www = new WWW(url);
                PlayerPrefs.SetFloat("WR" + currentLevel.ToString(), levelTimer);
                wrForLevel = levelTimer;
                setPB = true;
                updateLevelText();
            }
            if (pbForLevel == 0 || levelTimer < pbForLevel)
            {
                PlayerPrefs.SetFloat("PB" + currentLevel.ToString(), levelTimer);
                pbForLevel = levelTimer;
                setPB = true;
                
                updateLevelText();
            }


        }
    }
    void DeletedCoin()
    {
        coinCount--;
        coinTotal--;
    }

    // Update is called once per frame
    void Update () {

        if (hWasZero && Input.GetButton("Reset"))
        {
            if (Input.GetAxisRaw("Horizontal") < 0 && currentLevel > 1)
            {
                currentLevel--;
                buildMap(currentLevel);
            }
            if (Input.GetAxisRaw("Horizontal") > 0)
            {
                currentLevel++;
                buildMap(currentLevel);
            }
        }

        hWasZero = Input.GetAxisRaw("Horizontal") == 0;

        canMakeNoise = true;
        if (!cleared)
        {
            if(levelTimer > 0 || !Input.GetButton("Reset")) levelTimer += Time.deltaTime;
            timerText.text = prettyTime(levelTimer, true);
            timerText.color = new Color(1, 1, 1, 0.5f);
            if (levelTimer < 0) timerText.text = "";
        }
        else
        {
            finishTimer -= Time.deltaTime;
            timerText.text = prettyTime(levelTimer);
            timerText.color = new Color(1, 1, 0.5f, 1);
            if (finishTimer <= 0)
            {
                currentLevel++;
                //GameObject.FindGameObjectWithTag("Player").SendMessage("ResetPosition");
                buildMap(currentLevel);
            }
        }

        
        float readyProgress = levelTimer / -countdownLength;
        if (readyProgress * 3 < countdown && countdown >= 0)
        {

            if (countdown == 0)
            {
                GetComponent<AudioSource>().PlayOneShot(startSound);
                
                GameObject.Instantiate(finishRippleProto, new Vector3(player.transform.position.x, player.transform.position.y, -7), Quaternion.identity);
            }
            else
            {
                GetComponent<AudioSource>().PlayOneShot(tickSound);
                GameObject.Instantiate(normalRippleProto, new Vector3(player.transform.position.x, player.transform.position.y, -7), Quaternion.identity);
            }
            countdown--;
        }

        if (Input.GetButtonDown("Reset")) {
            //GameObject.FindGameObjectWithTag("Player").SendMessage("ResetPosition");
            buildMap(currentLevel);
        }
        if (Input.GetKeyDown(KeyCode.BackQuote))
        {
            Camera.main.orthographic = !Camera.main.orthographic;
        }
        if (Input.GetKeyDown(KeyCode.Escape)) Application.Quit();
	}
}
